@@ -20,7 +20,7 @@ from skimage.filters import gaussian
 from six.moves import reduce
 from matplotlib import cm
 from scipy import spatial
-from skimage.filters import threshold_local, threshold_otsu
+from skimage.filters import threshold_local, threshold_otsu, threshold_mean
 from skimage.segmentation import find_boundaries
 import matplotlib.pyplot as plt
 from scipy.ndimage.morphology import binary_fill_holes
@@ -147,15 +147,15 @@ def SimplePrediction(x, UnetModel, StarModel, n_tiles = (2,2), UseProbability =
                 
                       return SmartSeeds
 
-def crappify_flou_G_P(x, y, mu, sigma, savedirx, savediry, name):
+def crappify_flou_G_P(x, y, lam, savedirx, savediry, name):
     x = x.astype('float32')
-    gaussiannoise = np.random.normal(mu, sigma, x.shape)
+    gaussiannoise = np.random.poisson(lam, x.shape)
     x = x + gaussiannoise 
         
     #add noise to original image
-    imwrite(savedirx + '/' + name + 'pg' + str(mu) + str(sigma) + '.tif', x.astype('float32'))    
+    imwrite(savedirx + '/' + name + 'pg' + str(lam) + '.tif', x.astype('float32'))    
     #keep the label the same
-    imwrite(savediry + '/' + name + 'pg' + str(mu) + str(sigma) + '.tif', y.astype('uint16'))     
+    imwrite(savediry + '/' + name + 'pg' + str(lam)  + '.tif', y.astype('uint16'))     
 
 
 def _fill_label_holes(lbl_img, **kwargs):
@@ -473,8 +473,8 @@ def SmartSeedPredictionSliced(SaveDir, fname, UnetModel, StarModel, NoiseModel =
     imwrite((UNETResults + Name+ '.tif' ) , BinaryTime.astype('uint8'))   
     
     
-def SmartSeedPrediction3D( SaveDir, fname,  UnetModel, StarModel, NoiseModel = None, min_size_mask = 100, min_size = 100, 
-n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projection = False, UseProbability = True, filtersize = 0, globalthreshold = 1.0E-5, extent = 0):
+def SmartSeedPrediction3D( SaveDir, fname,  UnetModel, StarModel, NoiseModel = None, min_size_mask = 100, min_size = 100, max_size = 100000,
+n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projection = False, UseProbability = True, filtersize = 0, globalthreshold = 1.0E-5, extent = 0, seedpool = True):
     
     
     
@@ -515,8 +515,9 @@ n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projec
     SizedMask[:, :Mask.shape[1], :Mask.shape[2]] = Mask
     imwrite((UNETResults + Name+ '.tif' ) , SizedMask.astype('uint16')) 
     print('Stardist segmentation on Image')  
-    SmartSeeds, ProbabilityMap, StarImage, Markers = STARPrediction3D(gaussian_filter(image,filtersize), StarModel,  n_tiles, MaskImage = Mask, UseProbability = UseProbability, smartcorrection = smartcorrection, globalthreshold = globalthreshold, min_size = min_size, extent = extent)
+    SmartSeeds, ProbabilityMap, StarImage, Markers = STARPrediction3D(gaussian_filter(image,filtersize), StarModel,  n_tiles, MaskImage = Mask, UseProbability = UseProbability, smartcorrection = smartcorrection, globalthreshold = globalthreshold, min_size = min_size, extent = extent, seedpool = seedpool)
     SmartSeeds= remove_small_objects(SmartSeeds.astype('uint16'), min_size = min_size)
+    SmartSeeds= remove_big_objects(SmartSeeds.astype('uint16'), max_size = max_size)
     SmartSeeds = fill_label_holes(SmartSeeds.astype('uint16'))
     SmartSeeds = RemoveLabels(SmartSeeds) 
     SizedSmartSeeds[:, :SmartSeeds.shape[1], :SmartSeeds.shape[2]] = SmartSeeds
@@ -669,7 +670,7 @@ def UNETPrediction3D(image, model, n_tiles, axis):
     Segmented = model.predict(image, axis, n_tiles = n_tiles)
     
     try:
-       thresh = threshold_otsu(Segmented)
+       thresh = threshold_mean(Segmented)
        Binary = Segmented > thresh
     except:
         Binary = Segmented > 0
@@ -692,7 +693,7 @@ def RemoveLabels(LabelImage, minZ = 2):
                     LabelImage[LabelImage == regionlabel] = 0
     return LabelImage                
 
-def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection = None, UseProbability = True, globalthreshold = 1.0E-5, min_size = 100, extent = 0):
+def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection = None, UseProbability = True, globalthreshold = 1.0E-5, min_size = 100, extent = 0, seedpool = True):
     
     copymodel = model
     image = normalize(image, 1, 99.8, axis = (0,1,2))
@@ -740,7 +741,7 @@ def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection =
 
     
     print('Doing Watershedding')      
-    Watershed, Markers = WatershedwithMask3D(MaxProjectDistance.astype('uint16'), StarImage.astype('uint16'), MaskImage.astype('uint16'), grid, extent )
+    Watershed, Markers = WatershedwithMask3D(MaxProjectDistance.astype('uint16'), StarImage.astype('uint16'), MaskImage.astype('uint16'), grid, extent,seedpool )
     Watershed = fill_label_holes(Watershed.astype('uint16'))
   
        
@@ -793,7 +794,7 @@ def Conditioncheck(centroid, boxA, p, ndim, extent):
       return condition     
     
 
-def WatershedwithMask3D(Image, Label,mask, grid, extent = 0): 
+def WatershedwithMask3D(Image, Label,mask, grid, extent = 0, seedpool = True): 
     properties = measure.regionprops(Label, Image) 
     binaryproperties = measure.regionprops(label(mask), Image) 
     
@@ -804,7 +805,8 @@ def WatershedwithMask3D(Image, Label,mask, grid, extent = 0):
     Binarybbox = [prop.bbox for prop in binaryproperties]
     Coordinates = sorted(Coordinates , key=lambda k: [k[0], k[1], k[2]]) 
     
-    if len(Binarybbox) > 0:    
+    if seedpool:
+      if len(Binarybbox) > 0:    
             for i in range(0, len(Binarybbox)):
                 
                 box = Binarybbox[i]
